import boto3

import os

DEFAULT_CLIENT = boto3.client('ssm')

class SSMAdapter():
    def get(self, key, scope=None, app_name=None, client=DEFAULT_CLIENT):
        prefix = "/application"
        key_scope = scope if scope else app_name
        print(f"Attempting to read key {prefix}/{key_scope}/{key}")
        try:
            response = client.get_parameter(Name=f"{prefix}/{key_scope}/{key}", WithDecryption=True)
            return response['Value']
        except client.exceptions.ParameterNotFound:
            return None



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements/prod.txt') as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name="blinkistconfig",
    version="0.0.2",
    author="Peter Shoukry",
    author_email="peter@blinkist.com",
    description="Adapter based configuration handler (supports ENV and AWS SSM).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/blinkist/blinkist-config-python",
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    install_requires=requirements,
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



import blinkistconfig.adapters
from blinkistconfig import errors

class Factory:
    @staticmethod
    def by(type):
        adapter = f"{type}Adapter"
        try:
            return getattr(blinkistconfig.adapters, f"{type}Adapter")()
        except AttributeError:
            raise errors.AdapterNotFoundError(f"Adapter {adapter} not found") from None



from blinkistconfig import adapters
from blinkistconfig import errors

class Config:
    """
    Config represents a single configuration store. The interface is kept minimal
    with a single method to get a specific key from the store.

    ## Example
        config = Config(env="production", app_name="my_app", adapter="SSM")
        config.get("a/key")
    """

    def __init__(self, env, adapter_type, app_name):
        self.app_name = app_name
        self.env = env
        self.adapter_type = adapter_type

    # The *args is passed that way to handle None default values properly
    def get(self, key, *args, scope=None):
        """
        Returns the value of the key from the store or the default value if store
        fails
        """
        self._validate_params(*args)
        from_adapter = self._adapter().get(key, scope=scope, app_name=self.app_name)

        if from_adapter is None:
            return self._value_missing(key, *args, scope=scope)
        else:
            return from_adapter

    def _has_default(self, *args):
        return len(args) == 1

    def _default_value(self, *args):
        return args[0]

    def _validate_params(self, *args):
        args_length = len(args)
        error_message = f"wrong number of arguments"

        if args_length not in [0, 1]:
            raise ValueError(error_message)

    def _adapter(self):
        self.adapter = getattr(self, "adapter", adapters.Factory.by(self.adapter_type))
        return self.adapter

    def _handle_error(self, key, scope):
        raise errors.ValueMissingError(f"key: {key} has no value in {scope}")

    def _value_missing(self, key, *args, scope):
        if self._has_default(*args):
            return self._default_value(*args)
        self._handle_error(key, scope)
